@@ -2,13 +2,15 @@ from dtschema.lib import (
     add_schema_path,
     load_schema,
     load,
-    process_schemas,
-    set_schema,
+    set_schemas,
     DTValidator,
     format_error,
     extract_compatibles,
     extract_types,
-    get_prop_types,
+    get_undocumented_compatibles,
+    property_get_type,
+    property_get_type_dim,
+    property_has_fixed_dimensions,
     sized_int,
 )
 
