@@ -11,9 +11,6 @@ from libfdt import QUIET_NOTFOUND
 
 import dtschema
 
-props = {}
-pat_props = {}
-
 u8 = struct.Struct('B')
 s8 = struct.Struct('b')
 u16 = struct.Struct('>H')
@@ -79,58 +76,54 @@ def prop_value(nodename, p):
     if nodename in {'__fixups__', 'aliases'}:
         return data[:-1].decode(encoding='ascii').split('\0')
 
-    if p.name in props:
-        v = props[p.name]
-        if {'string', 'string-array'} & set(v['type']):
-            #print(p.name, v)
+    prop_types = set(dtschema.property_get_type(p.name))
+    prop_types -= {'node'}
+
+    # Filter out types impossible for the size of the property
+    if len(prop_types) > 1:
+        if len(p) > 4:
+            prop_types -= {'int32', 'uint32'}
+        if len(p) > 2:
+            prop_types -= {'int16', 'uint16'}
+        if len(p) > 1:
+            prop_types -= {'int8', 'uint8'}
+        else:
+            prop_types -= {'flag'}
+
+    if len(prop_types) > 1:
+        if {'string', 'string-array'} & prop_types:
             str = bytes_to_string(data)
             if str:
                 return str
             # Assuming only one other type
             try:
-                fmt = set(v['type']).difference({'string', 'string-array'}).pop()
+                fmt = prop_types.difference({'string', 'string-array'}).pop()
             except:
                 return data
-        elif len(v['type']):
-            fmt = v['type'][0]
-            # properties like ranges can be boolean or have a value
-            if fmt == 'flag' and len(data) and len(v['type']) > 1:
-                fmt = v['type'][1]
-            if (fmt.endswith('matrix') or fmt == 'phandle-array') and 'dim' in v:
-                dim = v['dim']
-            #print(p.name, fmt)
-    else:
-        for pat, v in pat_props.items():
-            if v['regex'].search(p.name):
-                #print(p.name, v, file=sys.stderr)
-                if fmt and fmt != v['type'][0]:
-                    #print('Multiple regex match with differring types:', fmt, v['type'][0], pat, file=sys.stderr)
-                    if len(data) > 4 and '-' in fmt:
-                        continue
-                fmt = v['type'][0]
-                fmt_base = fmt.split('-', 1)[0]
-                if fmt_base in type_format:
-                    if (fmt.endswith('matrix') or fmt == 'phandle-array') and 'dim' in v:
-                        dim = v['dim']
-                    continue
-                elif 'string' in v['type'][0]:
-                    return data[:-1].decode(encoding='ascii').split('\0')
         else:
-            if not fmt:
-                # Primarily for aliases properties
-                try:
-                    s = data.decode(encoding='ascii')
-                    if s.endswith('\0'):
-                        s = s[:-1]
-                        if s.isprintable():
-                            return [s]
-                except:
-                    pass
-                if not len(data) % 4:
-                    fmt = 'uint32-array'
-                else:
-                    #print(p.name + ': no type found', file=sys.stderr)
-                    return data
+            #print(p.name + ': multiple types found', file=sys.stderr)
+            fmt = None
+    elif len(prop_types) == 1:
+        fmt = prop_types.pop()
+
+    if not fmt:
+        # Primarily for aliases properties
+        try:
+            s = data.decode(encoding='ascii')
+            if s.endswith('\0'):
+                s = s[:-1]
+                if s.isprintable():
+                    return [s]
+        except:
+            pass
+        if not len(data) % 4:
+            fmt = 'uint32-array'
+        else:
+            #print(p.name + ': no type found', file=sys.stderr)
+            return data
+
+    if fmt.startswith('string'):
+        return data[:-1].decode(encoding='ascii').split('\0')
 
     if fmt == 'flag':
         if len(data):
@@ -158,6 +151,8 @@ def prop_value(nodename, p):
         for i in type_struct.iter_unpack(data):
             val_int += [dtschema.sized_int(i[0], size=(type_struct.size * 8))]
 
+    if 'matrix' in fmt or 'phandle-array' in fmt:
+        dim = dtschema.property_get_type_dim(p.name)
     if dim:
         if max(dim[1]) and dim[1][0] == dim[1][1]:
             stride = dim[1][1]
@@ -254,7 +249,8 @@ def fdt_scan_node(fdt, nodename, offset):
 
 
 phandle_args = {
-    # phandle+args properties which don't match standard 'foos' and '#foo-cells' pattern
+    # phandle+args properties with fixed arg size or which don't match standard
+    # 'foos' and '#foo-cells' pattern
     'assigned-clocks': '#clock-cells',
     'assigned-clock-parents': '#clock-cells',
     'cooling-device': '#cooling-cells',
@@ -262,6 +258,9 @@ phandle_args = {
     'interconnects': '#interconnect-cells',
     'mboxes': '#mbox-cells',
     'sound-dai': '#sound-dai-cells',
+    'msi-parent': '#msi-cells',
+    'msi-ranges': '#interrupt-cells',
+    'gpio-ranges': 3,
 
     'nvmem-cells': None,
     'memory-region': None,
@@ -269,7 +268,9 @@ phandle_args = {
 
 
 def _get_cells_size(node, cellname):
-    if cellname in node:
+    if isinstance(cellname, int):
+        return cellname
+    elif cellname in node:
         return node[cellname][0][0]
     else:
         return 0
@@ -302,8 +303,6 @@ def _get_phandle_arg_size(prop_path, idx, cells, cellname):
         return 0
 
     node = phandles[phandle]
-    if cellname not in node:
-        return 0
 
     return _get_cells_size(node, cellname) + 1
 
@@ -313,14 +312,16 @@ def fixup_phandles(dt, path=''):
         if isinstance(v, dict):
             fixup_phandles(v, path=path + '/' + k)
             continue
-        elif not k in props or not {'phandle-array'} & set(props[k]['type']):
+        elif not {'phandle-array'} & set(dtschema.property_get_type(k)):
+            continue
+        elif dtschema.property_has_fixed_dimensions(k):
             continue
         elif not isinstance(v, list) or (len(v) > 1 or not isinstance(v[0], list)):
             # Not a matrix or already split, nothing to do
             continue
         elif k in phandle_args:
             cellname = phandle_args[k]
-        elif k.endswith('s'):
+        elif k.endswith('s') and 'gpio' not in k:
             cellname = '#' + k[:-1] + '-cells'
             #print(k, v)
             i = _get_phandle_arg_size(path + ':' + k, 0, v[0], cellname)
@@ -340,6 +341,10 @@ def fixup_phandles(dt, path=''):
                 #print(k, v)
                 break
 
+            # Special case for msi-ranges which has an additional span cell
+            if k == 'msi-ranges':
+                cells += 1
+
             # Special case for interconnects which is pairs of phandles+args
             if k == 'interconnects':
                 cells += _get_phandle_arg_size(path + ':' + k, i + cells, val[i + cells:], cellname)
@@ -460,13 +465,6 @@ def fixup_addresses(dt, ac, sc):
 
 
 def fdt_unflatten(dtb):
-    p = dtschema.get_prop_types()
-    global props
-    global pat_props
-
-    props = p[0]
-    pat_props = p[1]
-
     fdt = libfdt.Fdt(dtb)
 
     offset = fdt.first_subnode(-1, QUIET_NOTFOUND)
