@@ -987,19 +987,19 @@ def property_get_type(propname):
     if not props:
         props, pat_props = get_prop_types()
 
-    type = []
+    type = set()
     if propname in props:
         for v in props[propname]:
             if v['type']:
-                type += [v['type']]
+                type.add(v['type'])
     if len(type) == 0:
         for v in pat_props.values():
             if v[0]['type'] and v[0]['type'] not in type and v[0]['regex'].search(propname):
-                type += [v[0]['type']]
+                type.add(v[0]['type'])
 
     # Don't return 'node' as a type if there's other types
     if len(type) > 1 and 'node' in type:
-        type.remove('node')
+        type -= {'node'}
     return type
 
 
@@ -1022,10 +1022,23 @@ def property_get_type_dim(propname):
     return None
 
 
+def property_has_fixed_dimensions(propname):
+    dim = property_get_type_dim(propname)
+    if dim and dim[0][0] == dim[0][1] or dim[1][0] == dim[1][1]:
+        return True
+
+    return False
+
+
 def load(filename, line_number=False):
-    if filename.endswith('.dtb'):
-        with open(filename, 'rb') as f:
-            return [ dtschema.dtb.fdt_unflatten(f.read()) ]
+    try:
+        if not filename.endswith('.yaml'):
+            with open(filename, 'rb') as f:
+                return [ dtschema.dtb.fdt_unflatten(f.read()) ]
+    except:
+        if filename.endswith('.dtb'):
+            raise
+
     with open(filename, 'r', encoding='utf-8') as f:
         if line_number:
             return rtyaml.load(f.read())
@@ -1127,21 +1140,16 @@ handlers = {"http": http_handler}
 
 
 def typeSize(validator, typeSize, instance, schema):
-    if not isinstance(instance[0], list):
-        return
-    if isinstance(instance[0][0], sized_int):
+    try:
         size = instance[0][0].size
-    else:
+    except:
         size = 32
 
     if typeSize != size:
         yield jsonschema.ValidationError("size is %r, expected %r" % (size, typeSize))
 
 
-DTVal = jsonschema.validators.extend(jsonschema.Draft201909Validator, {'typeSize': typeSize})
-
-
-class DTValidator(DTVal):
+class DTValidator():
     '''Custom Validator for Devicetree Schemas
 
     Overrides the Draft7 metaschema with the devicetree metaschema. This
@@ -1151,11 +1159,12 @@ class DTValidator(DTVal):
     '''
     resolver = jsonschema.RefResolver('', None, handlers=handlers)
     format_checker = jsonschema.FormatChecker()
+    DTVal = jsonschema.validators.extend(jsonschema.Draft201909Validator, {'typeSize': typeSize}, version='DT')
 
     def __init__(self, *args, **kwargs):
         kwargs.setdefault('resolver', self.resolver)
         kwargs.setdefault('format_checker', self.format_checker)
-        super().__init__(*args, **kwargs)
+        self.validator = self.DTVal(*args, **kwargs)
 
     @classmethod
     def annotate_error(self, error, schema, path):
@@ -1165,7 +1174,7 @@ class DTValidator(DTVal):
         for e in error.context:
             self.annotate_error(e, schema, path + e.schema_path)
 
-        scope = self.ID_OF(schema)
+        scope = schema['$id']
         self.resolver.push_scope(scope)
         ref_depth = 1
 
@@ -1201,22 +1210,32 @@ class DTValidator(DTVal):
     @classmethod
     def iter_schema_errors(cls, schema):
         meta_schema = cls.resolver.resolve_from_url(schema['$schema'])
-        for error in cls(meta_schema).iter_errors(schema):
-            cls(meta_schema).annotate_error(error, meta_schema, error.schema_path)
+        val = cls.DTVal(meta_schema, resolver=cls.resolver)
+        for error in val.iter_errors(schema):
+            cls.annotate_error(error, meta_schema, error.schema_path)
             error.linecol = get_line_col(schema, error.path)
             yield error
 
     def iter_errors(self, instance, _schema=None):
-        for error in super().iter_errors(instance, _schema):
+        for error in self.validator.iter_errors(instance, _schema):
             error.linecol = get_line_col(instance, error.path)
             error.note = None
             error.schema_file = None
             yield error
 
+    def validate(self, *args, **kwargs):
+        for error in self.iter_errors(*args, **kwargs):
+            raise error
+
+    def is_valid(self, instance):
+        error = next(self.iter_errors(instance), None)
+        return error is None
+
     @classmethod
     def check_schema(cls, schema):
         meta_schema = cls.resolver.resolve_from_url(schema['$schema'])
-        for error in cls(meta_schema).iter_errors(schema):
+        val = cls.DTVal(meta_schema, resolver=cls.resolver)
+        for error in val.iter_errors(schema):
             raise jsonschema.SchemaError.create_from(error)
         fixup_schema(schema)
 
@@ -1233,7 +1252,7 @@ class DTValidator(DTVal):
 
     @classmethod
     def check_schema_refs(self, filename, schema):
-        scope = self.ID_OF(schema)
+        scope = schema['$id']
         if scope:
             self.resolver.push_scope(scope)
 
@@ -1313,7 +1332,7 @@ def format_error(filename, error, prefix="", nodename=None, verbose=False):
                 msg += '\n' + format_error(filename, suberror, prefix=prefix+"\t", nodename=nodename, verbose=verbose)
             elif suberror.message not in msg:
                 msg += '\n' + prefix + '\t' + suberror.message
-                if suberror.note and suberror.note != error.note:
+                if hasattr(suberror, 'note') and suberror.note and suberror.note != error.note:
                     msg += '\n\t\t' + prefix + 'hint: ' + suberror.note
 
     elif error.schema_path[-1] == 'oneOf':
